package common;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import processing.FileDirPropertiesProc;

public class Support
{
	private static String fDateStamp = null;

	public static int getBinderStartIndexInPath(String[] pathArray) throws Exception
	{
		int binderStart = 0;
		for (int n = pathArray.length - 1; n >= 0; n--)
		{
			String path = Utilities.joinArray(pathArray, 0, n, "/");
			File file = new File(path);
			String directoryName = file.getName();
			if (directoryName.endsWith("_"))
			{
				binderStart = n;
				break;
			}
			binderStart = -1; // Indicate binder not found
		}
		if (binderStart == -1)
			throw new BinderException(
					"Error, Binder not found\nMake sure the root binder directory is marked with '_' at the end of the Binder directory name.");
		if (OperatingSystem.isWindows())
		{
			if (binderStart < 1)
				throw new BinderException(
						"Error, illegal binder directory \n Make sure the root binder directory as marked with '_' at the end of the binder's name \n is at least two levels down from the root (drive) in the file system.");
		} else
		// Linux or the Mac
		{
			if (binderStart < 1)
				throw new BinderException(
						"Error, illegal binder directory \n Make sure the root binder directory as marked with '_' at the end of the Binder's name  name\n is at least two levels down from the root of the file system.");
		}
		return (binderStart);
	}

	public static String getDateStamp()
	{
		// It is assumed that the date is the same for the  same instance of the program
		if (fDateStamp == null)
		{
			DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
			Date date = new Date();
			fDateStamp = dateFormat.format(date);
		}
		return (fDateStamp);
	}

	public static String getBinderName(String defaultName)
	{
		String binderName = System.getProperty("binder.alias");
		if ((binderName != null) && (binderName.equals("none") == false))
			return (binderName);
		if (defaultName == null)
			return (null);
		binderName = defaultName.replaceAll("_", " ").trim();
		return (binderName);
	}

	public static String getPageName(String sourcePath, String defaultName)
	{
		String pageName = FileDirPropertiesProc.getInstance().getAliasProperty(sourcePath);
		if ((pageName != null) && (pageName.equals("none") == false))
			return (pageName);
		if (defaultName == null)
			return (null);
		pageName = defaultName.replaceAll("\\.[a-zA-Z0-9]{2,4}$", "");
		pageName = pageName.replaceAll("_", " ");
		return (pageName);
	}

	public static String getStyleSheetTemplatePath(String sourceDirectoryPath, String templateName) throws Exception
	{
		String sourceBinderDirectoryPath = System.getProperty("source.binder.directory.path");
		String tempDirectoryPath = sourceDirectoryPath;
		for (int n = 0; n < 10; n++)
		{
			tempDirectoryPath = tempDirectoryPath.replaceAll("\\\\", "/");
			String cssTemplatePath = tempDirectoryPath + "/_resource/" + templateName + "-template.txt";
			File cssTemplateFile = new File(cssTemplatePath);
			if (cssTemplateFile.exists())
				return (cssTemplatePath);
			if (tempDirectoryPath.equals(sourceBinderDirectoryPath))
				break;
			tempDirectoryPath = (new File(tempDirectoryPath)).getParent();
		}
		return (null);
	}

}
