package common;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

public class Utilities
{

	public static String joinArray(String[] array, int start, int end, String separator) throws Exception
	{
		if (array == null)
			throw new BinderException("Error in Join:\narray null");
		StringBuffer buffer = new StringBuffer();
		for (int n = start; n <= end; n++)
		{
			buffer.append(array[n]);
			if (n < end)
				buffer.append(separator);
		}
		String string = buffer.toString();
		return (string);
	}

	public static String[] splitPath(String path)
	{
		if (OperatingSystem.isWindows() == false) // Linux or the Mac
		{
			// Remove root '/' for correct split operation
			path = path.replaceAll("^\\/", "");
		}
		String[] fileArray = path.split("[\\/\\\\]+");
		return (fileArray);
	}

	public static String joinPathArray(String[] array, int start, int end) throws Exception
	{
		String path = joinArray(array, start, end, "/");
		if (OperatingSystem.isWindows() == false) // Linux or the Mac
		{
			if (start == 0) // Add the root)
				path = "/" + path;
		}
		return (path);
	}

	/*
	 * Note that Java does not mark or recognize UTF-8 with a BOM It is not
	 * clear if a file is read with utf-8 bom - needs investigation if this
	 * method is to be used
	 */
	public static String fileToStringUTF8(String filePath) throws IOException
	{
		BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(new FileInputStream(filePath), "UTF8"));
		StringBuffer stringBuffer = new StringBuffer();
		String line;
		while ((line = bufferedReader.readLine()) != null)
		{
			stringBuffer.append(line + "\n");
		}
		bufferedReader.close();
		return (stringBuffer.toString());
	}

	public static String fileToString(String filePath) throws IOException
	{
		return (fileToStringBuffer(filePath).toString());
	}

	public static StringBuffer fileToStringBuffer(String filePath) throws IOException
	{
		BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(new FileInputStream(filePath)));
		StringBuffer stringBuffer = new StringBuffer();
		String line;
		while ((line = bufferedReader.readLine()) != null)
		{
			stringBuffer.append(line + "\n");
		}
		bufferedReader.close();
		return (stringBuffer);
	}

	/*
	 * Note that Java does not mark or recognize UTF-8 with a BOM This does not
	 * display correctly in some editor, unless it is identified by a bom
	 * (optional. in this method
	 */
	public static void stringToFileUTF8(String filePath, String string, boolean bomRequired) throws Exception
	{
		FileOutputStream fileOutputStream = new FileOutputStream(filePath);
		try
		{
			if (bomRequired)
			{
				byte[] bom = new byte[]
				{ (byte) 0xEF, (byte) 0xBB, (byte) 0xBF };
				fileOutputStream.write(bom);
			}
			Writer writer = new BufferedWriter(new OutputStreamWriter(fileOutputStream, "UTF8"));
			writer.write(string);
			writer.close();
		} finally
		{
			fileOutputStream.close();
		}
	}

	public static void stringToFile(String filePath, String string) throws IOException
	{
		Writer writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(filePath)));
		writer.write(string);
		writer.close();
	}

	public static void copyFile(File sourceFile, File targetFile) throws IOException, FileNotFoundException
	{
		if (sourceFile.equals(targetFile))
			return; // Can't copy to itself
		InputStream fis = new FileInputStream(sourceFile);
		OutputStream fos = new FileOutputStream(targetFile);
		try
		{
			byte[] buf = new byte[1024];
			int i = 0;
			while ((i = fis.read(buf)) != -1)
			{
				fos.write(buf, 0, i);
			}
		} finally
		{
			fis.close();
			fos.close();
		}
	}

	/**
	 * Delete directory by deleting all files and subdirectories under dir.
	 * Returns true if all deletions were successful. If a deletion fails, the
	 * method stops attempting to delete and returns false.
	 * 
	 * @param directory -
	 *            the directory to delete.
	 * @return - True if successfull False if not.
	 */
	public static boolean deleteDirectory(File directory)
	{
		if (directory.isDirectory())
		{
			String[] children = directory.list();
			for (int i = 0; i < children.length; i++)
			{
				deleteDirectory(new File(directory, children[i]));
			}
		}
		// The directory should now be empty so attempt to delete it
		return directory.delete();
	}

}
