package config;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

import java.io.File;

import common.BinderException;

import formats.BinderProperties;
import formats.FileDirProperties;

public class Settings
{
	public final static String VERSION = "MarkupBinder Version 0.6 beta - 28 October 2010";
	public final static String COPYRIGHT = "Copyright (c) Isaac Porat 2010.  All rights reserved.";

	public final static String ARGS_SEP = "qQq"; // Separator between Pandoc arguments to allow reliable separation to an array
	public final static String WEB_BINDER_INDICATOR = "-Web";
	public final static String FILES_EXCLUDED_FROM_CLEANING = "md"; // File extensions separated by commas
	public final static String CONTENT_FILE_EXTENSION = "md";
	public final static String INSERT_FILE_SUFFIX = "-insert.md";

	public final static String TREE_FRONT_CSS_NAME = "zz-tree-front";
	public final static String TREE_SECTION_CSS_NAME = "zz-tree-section";
	public final static String TREE_CONTENT_CSS_NAME = "zz-tree-content";
	public final static String DOCUMENT_INTERNAL_CSS_NAME = "zz-document";

	public final static String BINDER_PROPERTIES_FILE_NAME = "binder-properties";
	public final static String DIRECTORY_PROPERTIES_FILE_NAME = "folder-properties";

	private final static String DEFAULT_TREE_SECTION_TITLE = "Section";
	private final static String DEFAULT_TREE_CONTENT_TITLE = "file";

	private final static String DEFAULT_DOCUMENT_TITLE = "file";

	private final static String DEFAULT_DEBUG_MODE = "false";
	private final static String DEFAULT_BINDER_AUTHOR = "none";

	public static void setEditModeProperties()
	{
		// 'none' - using local directory
		System.setProperty("tree.target.base.directory.path", "none");
		// false - leave spaces in directory and file names
		System.setProperty("tree.web.mode", "false");
	}

	public static void setWebModeProperties(String targetBaseHTMLDirectoryPath)
	{
		System.setProperty("tree.target.base.directory.path", targetBaseHTMLDirectoryPath);
		// Replace spaces with underlines in directory and file names
		System.setProperty("tree.web.mode", "true");
	}

	public static void setBinderProperties(String directoryPath, String binderName) throws Exception
	{
		String propertiesDirectoryPath = directoryPath + "/_resource/_properties";
		String propertiesPath = propertiesDirectoryPath + "/" + BINDER_PROPERTIES_FILE_NAME + ".txt";
		File binderPropertiesFile = new File(propertiesPath);
		BinderProperties binderProperties = new BinderProperties();
		if (binderPropertiesFile.exists() == false)
		{
			File propertiesDirectoryFile = new File(propertiesDirectoryPath);
			if (propertiesDirectoryFile.exists() == false)
			{
				if (propertiesDirectoryFile.mkdirs() == false)
					throw new BinderException("Error in setBinderProperties:\n Can't create " + propertiesDirectoryPath);
			}
			binderProperties.addComment("Binder properties - change these properties to suit your needs");
			binderProperties.addComment("Important - do not change the key (the left side of the '=' sign");
			binderProperties.addComment("Modify only the value - on the right side of the '=' sign");
			binderProperties.addComment("");
			// Add title properties
			binderProperties.addComment("The title of the front binder page as displayed in the browser");
			binderProperties.addProperty("tree.front.title", binderName + " - Main Index");
			binderProperties.addComment("The title of the section pages as displayed in the browser");
			binderProperties.addProperty("tree.section.title", DEFAULT_TREE_SECTION_TITLE);
			binderProperties.addComment("The title of the content pages as displayed in the browser");
			binderProperties.addComment("file - title file name, none - no title, any other value title equals value.");
			binderProperties.addProperty("tree.content.title", DEFAULT_TREE_CONTENT_TITLE);
			binderProperties.addComment("");
			binderProperties.addComment("The title of the document pages as displayed in the browser");
			binderProperties.addComment("file - title file name, none - no title, any other value title equals value.");
			binderProperties.addProperty("document.title", DEFAULT_DOCUMENT_TITLE);
			binderProperties
					.addComment("Automatically derive a document name from the file name and add it to the document");
			binderProperties.addComment("true or false");
			binderProperties.addProperty("document.name.auto", "false");
			binderProperties.addComment("Automatically derived name heading level");
			binderProperties.addComment("Heading level values between 1 - 6");
			binderProperties.addProperty("document.name.auto.heading.level", "3");
			binderProperties.addComment("");
			binderProperties.addComment("Binder name alias");
			binderProperties.addProperty("binder.alias", "none");
			binderProperties.addComment("");
			binderProperties.addComment("Binder's Author - none or author name");
			binderProperties.addProperty("binder.author", DEFAULT_BINDER_AUTHOR);
			binderProperties.addComment("");
			binderProperties.addComment("Debug Mode - true or false");
			binderProperties.addProperty("debug.mode", DEFAULT_DEBUG_MODE);
			try
			{
				binderProperties.save(propertiesPath);
			} catch (Exception ex)
			{
				throw new BinderException(
						"Error in setBinderProperties:\n properties file unable to save properties file "
								+ ex.toString());
			}
		}
		try
		{
			binderProperties.load(propertiesPath);
		} catch (Exception ex)
		{
			throw new BinderException("Error in setBinderProperties:\n unable to load properties file " + ex.toString());
		}
		binderProperties.propertiesToSystem();
	}

	public static void setFileDirProperties(String directoryPath, String fileDirName) throws Exception
	{
		String fileDirPath = directoryPath + "/" + fileDirName;
		File fileDirFile = new File(fileDirPath);
		if (fileDirFile.exists() == false)
			return;
		boolean directoryFlag = false;
		String propertiesDirectoryPath;
		String propertiesPath;
		if (fileDirFile.isDirectory())
		{
			propertiesDirectoryPath = fileDirPath + "/_resource/_properties";
			propertiesPath = propertiesDirectoryPath + "/" + DIRECTORY_PROPERTIES_FILE_NAME + ".txt";
			directoryFlag = true;
		} else
		{
			propertiesDirectoryPath = directoryPath + "/_resource/_properties";
			propertiesPath = propertiesDirectoryPath + "/" + fileDirName + "-properties.txt";
		}
		File propertiesFile = new File(propertiesPath);
		FileDirProperties fileDirProperties = new FileDirProperties();
		if (propertiesFile.exists() == false)
		{
			File propertiesDirectoryFile = new File(propertiesDirectoryPath);
			if (propertiesDirectoryFile.exists() == false)
			{
				if (propertiesDirectoryFile.mkdirs() == false)
					throw new BinderException("Error in setFileDirProperties:\n Can't create "
							+ propertiesDirectoryPath);
			}
			if (directoryFlag)
				fileDirProperties.addComment("Folder properties - change these properties to suit your needs");
			else
				fileDirProperties.addComment("File properties - change these properties to suit your needs");
			fileDirProperties.addComment("Important - do not change the key (the left side of the '=' sign");
			fileDirProperties.addComment("Modify only the value - on the right side of the '=' sign");
			fileDirProperties.addComment("");
			if (directoryFlag)
				fileDirProperties.addComment("Folder name alias");
			else
				fileDirProperties.addComment("File name alias");
			fileDirProperties.addProperty("alias", "none");
			try
			{
				fileDirProperties.save(propertiesPath);
			} catch (Exception ex)
			{
				throw new BinderException(
						"Error in setFileDirProperties:\n properties file unable to save properties file "
								+ ex.toString());
			}
		}
	}

}
