package formats;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

import common.Utilities;

/*
 * This class implements relative font size which is supposed
 * to provide better accessibility by ensuring font resizing in all browsers. 
 */
public class CSSWriter
{
	private StringBuffer fStringBuffer;
	private String fLS;

	public CSSWriter()
	{
		fStringBuffer = new StringBuffer();
		fLS = System.getProperty("line.separator");
	}

	public void write(String string)
	{
		fStringBuffer.append(string + fLS);
	}

	public void startSelector(String selector)
	{
		fStringBuffer.append(selector + fLS + "{" + fLS);
	}

	public void endSelector()
	{
		fStringBuffer.append("}" + fLS);
	}

	public void lineSelector(String select, String declarations)
	{
		fStringBuffer.append(select + "{" + declarations + "}" + fLS);
	}

	public void comment(String caption)
	{
		fStringBuffer.append("/* " + caption + " */" + fLS);
	}

	public void newLine()
	{
		fStringBuffer.append(fLS);
	}

	public void save(String targetPath) throws Exception
	{
		Utilities.stringToFile(targetPath, fStringBuffer.toString());
	}

	public void defaultColour()
	{
		fStringBuffer.append("color: #000000;" + fLS);
		fStringBuffer.append("background-color: #FFFFFF;" + fLS);
	}

	public void defaultDimensions()
	{
		fStringBuffer.append("width: 15.5cm;" + fLS);
		fStringBuffer.append("margin-left: 3cm;" + fLS);
		fStringBuffer.append("margin-right: 2.5cm;" + fLS);
		fStringBuffer.append("margin-top: 3cm;" + fLS);
	}

	public void defaultText()
	{
		fStringBuffer.append("text-align: left;" + fLS);
		fStringBuffer.append("text-decoration: none;" + fLS);
	}

	public void defaultFont()
	{
		fStringBuffer.append("font-size: 100%;" + fLS);
		fStringBuffer.append("font-weight: normal;" + fLS);
		fStringBuffer.append("font-style: normal;" + fLS);
	}

	public void defaultFontName()
	{
		fStringBuffer.append("font-family: Arial,Sans-serif;" + fLS);
	}

	public void makeDefaultExternal(String targetPath, String caption) throws Exception
	{
		comment(caption + " Style Sheet");
		startSelector("body");
		defaultColour();
		defaultDimensions();
		defaultText();
		defaultFontName();
		defaultFont();
		endSelector();
		newLine();
		startSelector("p,h1,h2,h3,h4,h5,h6,ul,ol,blockquote");
		defaultFontName();
		endSelector();
		newLine();
		comment("Elements font relative size");
		lineSelector("h1", "font-size: 2.5em");
		lineSelector("h2", "font-size: 2.2em");
		lineSelector("h3", "font-size: 1.9em");
		lineSelector("h4", "font-size: 1.6em");
		lineSelector("h5", "font-size: 1.4em");
		lineSelector("h6", "font-size: 1.2em");
		lineSelector("p,ul,ol,code,blockquote", "font-size: 1.2em");
		comment("'pre' size to ensure acceptable appearance  in both Firefox and IE");
		lineSelector("pre", "font-size: 1.35em");
		newLine();
		comment("Elements descendents to ensure the same font size as parents with Markdown conversion");
		lineSelector("ul ul", "font-size: 1.0em");
		lineSelector("ul p", "font-size: 1.0em");
		lineSelector("ol ol", "font-size: 1.0em");
		lineSelector("ol p", "font-size: 1.0em");
		lineSelector("blockquote p", "font-size: 1.0em");
		lineSelector("pre code", "font-size: 1.0em");

		save(targetPath);
	}

	public void makeDefaultInternal(String targetPath, String caption) throws Exception
	{
		write("<style type=\"text/css\">");
		write("<!--"); // Compatibility with older browsers
		comment(caption + " Style Sheet");
		startSelector("body");
		defaultColour();
		defaultDimensions();
		defaultText();
		defaultFontName();
		defaultFont();
		endSelector();
		newLine();
		startSelector("p,h1,h2,h3,h4,h5,h6,ul,ol,blockquote");
		defaultFontName();
		endSelector();
		newLine();
		comment("Elements font relative size");
		lineSelector("h1", "font-size: 2.5em");
		lineSelector("h2", "font-size: 2.2em");
		lineSelector("h3", "font-size: 1.9em");
		lineSelector("h4", "font-size: 1.6em");
		lineSelector("h5", "font-size: 1.4em");
		lineSelector("h6", "font-size: 1.2em");
		lineSelector("p,ul,ol,code,blockquote", "font-size: 1.2em");
		comment("'pre' size to ensure acceptable appearance  in both Firefox and IE");
		lineSelector("pre", "font-size: 1.35em");
		newLine();
		comment("Elements descendents to ensure the same font size as parents with Markdown conversion");
		lineSelector("ul ul", "font-size: 1.0em");
		lineSelector("ul p", "font-size: 1.0em");
		lineSelector("ol ol", "font-size: 1.0em");
		lineSelector("ol p", "font-size: 1.0em");
		lineSelector("blockquote p", "font-size: 1.0em");
		lineSelector("pre code", "font-size: 1.0em");
		write("-->"); // Compatibility with older browsers
		write("</style>");
		save(targetPath);
	}
}
