package formats;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

/*
 * This class does not implement Java Properties in the pure sense.
 * but rather loads Binder properties as a StringBuffer 
 * and then transfers to the Java System properties
 * It also implements a simple writer for creating an initial Binder default properties file. 
 */

import java.util.HashMap;
import java.util.Map;

import common.BinderException;
import common.Utilities;

public class FileDirProperties
{
	private StringBuffer fStringBuffer;
	private String fLS;
	private Map<String, String> fProperties;

	public FileDirProperties()
	{
		fStringBuffer = new StringBuffer();
		fLS = System.getProperty("line.separator");
		fProperties = new HashMap<String, String>();
	}

	public void addComment(String caption)
	{
		fStringBuffer.append("# " + caption + fLS);
	}

	public void addProperty(String key, String value)
	{
		if ((key == null) || (value == null))
			return;
		key = key.trim();
		value = value.trim();
		if ((key.equals("")) || (value.equals("")))
			return;
		fStringBuffer.append(key + "=" + value + fLS);
	}

	public void propertiesToMap() throws Exception
	{
		if (fStringBuffer.length() == 0)
			throw new BinderException("Error in method propertiesToSystem:\n StringBuffer empty");
		String string = fStringBuffer.toString();
		if (string == null)
			return;
		string = string.trim();
		if (string.equals(""))
			throw new BinderException("Error in method propertiesToSystem:\n no data found in properties file");
		String[] lines = string.split("\n");
		for (int n = 0; n < lines.length; n++)
		{
			lines[n] = lines[n].trim();
			if ((lines[n].equals("")) || (lines[n].startsWith("#")))
				continue;
			if (lines[n].contains("=") == false)
			{
				System.err.println("Error in line " + lines[n]
						+ " property must contain a key and value separated by an equals '=' sign, property ignored");
				continue;
			}
			String[] parts = lines[n].split("=");
			String key = parts[0].trim();
			String value = parts[1].trim();
			if ((key.equals("")) || (value.equals("")))
			{
				System.err.println("Error in line " + lines[n]
						+ " property must contain a key and value separated by an equals '=' sign, property ignored");
				continue;
			}
			fProperties.put(key, value);
		}
	}

	public Map <String, String> getProperties()
	{
		return (fProperties);
	}
	
	public String getProperty(String key)
	{
		return (fProperties.get(key));
	}

	public void load(String sourcePath) throws Exception
	{
		String string = Utilities.fileToString(sourcePath);
		fStringBuffer = new StringBuffer(string);
	}

	public void save(String targetPath) throws Exception
	{
		Utilities.stringToFile(targetPath, fStringBuffer.toString());
	}

}
