package formats;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

/*
 * This is the old css implementation using fixed size fonts - left for reference.
 * The new implementation of this class uses relative font size which is somewhat more difficult to implement
 * but suppose to provide better accessibility by ensuring font resizing for all browsers.
 */

import common.Utilities;

/*
 * This class implements fixed size fonts
 * This was replaced by a similar class which implements relative font size which suppose to ensure better accessibility
 * by ensuring font resizing in all browsers.
 */
public class FixedSizeFontsCSSWriter
{
	private StringBuffer fStringBuffer;
	private String fLS;

	public FixedSizeFontsCSSWriter()
	{
		fStringBuffer = new StringBuffer();
		fLS = System.getProperty("line.separator");
	}

	public void startSelector(String selector)
	{
		fStringBuffer.append(selector + fLS + "{" + fLS);
	}

	public void endSelector()
	{
		fStringBuffer.append("}" + fLS);
	}

	public void comment(String caption)
	{
		fStringBuffer.append("/* " + caption + " */" + fLS);
	}

	public void newLine()
	{
		fStringBuffer.append(fLS);
	}

	public void save(String targetPath) throws Exception
	{
		Utilities.stringToFile(targetPath, fStringBuffer.toString());
	}

	public void defaultColour()
	{
		fStringBuffer.append("color: #000000;" + fLS);
		fStringBuffer.append("background-color: #FFFFFF;" + fLS);
	}

	public void defaultDimensions()
	{
		fStringBuffer.append("width: 15.5cm;" + fLS);
		fStringBuffer.append("margin-left: 3cm;" + fLS);
		fStringBuffer.append("margin-right: 2.5cm;" + fLS);
		fStringBuffer.append("margin-top: 3cm;" + fLS);
	}

	public void defaultText()
	{
		fStringBuffer.append("text-align: left;" + fLS);
		fStringBuffer.append("text-decoration: none;" + fLS);
	}

	public void defaultFont()
	{
		fStringBuffer.append("font-family: Arial;" + fLS);
		fStringBuffer.append("font-size: 11pt;" + fLS);
		fStringBuffer.append("font-weight: normal;" + fLS);
		fStringBuffer.append("font-style: normal;" + fLS);
	}

	public void defaultFontName()
	{
		fStringBuffer.append("font-family: Arial;" + fLS);
	}

	public void makeDefault(String targetPath, String caption) throws Exception
	{
		comment(caption + " Style Sheet");
		startSelector("body");
		defaultColour();
		defaultDimensions();
		defaultText();
		defaultFont();
		endSelector();
		newLine();
		startSelector("p,h1,h2,h3,h4,h5,h6");
		defaultFontName();
		endSelector();
		save(targetPath);
	}

}
