package pandoc;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

import java.io.File;
import java.io.IOException;

import common.BinderException;

import config.Settings;

public class PandocExec
{
	private Process fProcess;

	public void run(String fromFormat, String toFormat, String[] sourcePathArray, String targetPath,
			String additionalArgs) throws Exception
	{
		try
		{
			String program = "pandoc";
			StringBuffer allSourcePathBuffer = new StringBuffer();
			for (int n = 0; n < sourcePathArray.length; n++)
			{
				allSourcePathBuffer.append((new File(sourcePathArray[n])).getPath() + Settings.ARGS_SEP);
			}
			String allSourcePath = allSourcePathBuffer.toString();
			//      Convert targetPath to OS specific directory separators
			targetPath = (new File(targetPath)).getPath();
			if (additionalArgs.equals("") == false)
				additionalArgs = Settings.ARGS_SEP + additionalArgs;
			String arguments = "-f" + Settings.ARGS_SEP + fromFormat + Settings.ARGS_SEP + "-t" + Settings.ARGS_SEP
					+ toFormat + Settings.ARGS_SEP + allSourcePath + Settings.ARGS_SEP + "-o" + Settings.ARGS_SEP
					+ targetPath + Settings.ARGS_SEP + additionalArgs;
			String command = program + Settings.ARGS_SEP + arguments;
			// Remove any double separators
			command = command.replaceAll(Settings.ARGS_SEP + Settings.ARGS_SEP, Settings.ARGS_SEP);
			// Remove any trailing separators
			command = command.replaceAll(Settings.ARGS_SEP + "$", "");
			String[] commandArray = command.split(Settings.ARGS_SEP);
			fProcess = Runtime.getRuntime().exec(commandArray);
			Thread.sleep(500);
			//       Capture STDERR and STDOUT.
			//        STDERR
			SStreamConsumer errorConsumer = new SStreamConsumer(fProcess.getErrorStream(), "error");
			//        STDOUT
			SStreamConsumer outputConsumer = new SStreamConsumer(fProcess.getInputStream(), "info");
			//        Start consumer threads
			//        process needs to start properly before StreamConsumer threads
			Thread.sleep(250);
			errorConsumer.start();
			outputConsumer.start();
		} catch (IOException ex)
		{
			String ls = System.getProperty("line.separator");
			throw new BinderException("Error in PandocExec:" + ls + ex.toString());
		}
	}

	public void destroy()
	{
		if (fProcess != null)
		{
			fProcess.destroy();
			try
			{
				Thread.sleep(500);
			} catch (InterruptedException ex)
			{
			}
			fProcess = null;
		}
	}

}
