package processing;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

import java.io.File;
import java.io.IOException;

import pandoc.PandocExec;

import common.BinderException;
import common.Report;
import common.Support;
import common.Utilities;

import config.Settings;
import formats.MarkdownWriter;

public class ContentProc
{
	private String fFromFormat;
	private String fToFormat;
	private String fAdditionalArgs = "";

	public void setFromFormat(String fromFormat)
	{
		fFromFormat = fromFormat;
	}

	public void setToFormat(String toFormat)
	{
		fToFormat = toFormat;
	}

	public void setAdditionalArgs(String additionalArgs)
	{
		fAdditionalArgs = additionalArgs;
	}

	public void convertFileFormat(String[] sourcePathArray, String targetPath) throws Exception
	{
		(new PandocExec()).run(fFromFormat, fToFormat, sourcePathArray, targetPath, fAdditionalArgs);
	}

	public void makeFileBefore(String sourceDirectoryPath, String sourcePath, String sourceFileName,
			BinderPathProc binderPathProc) throws Exception
	{
		MarkdownWriter markdownWriter = new MarkdownWriter();
		// Generate Pandoc header
		markdownWriter.addString("%");
		markdownWriter.newLine();
		String author = System.getProperty("binder.author");
		if ((author == null) || (author.equals("none")))
			author = "";
		markdownWriter.addString("% " + author);
		markdownWriter.newLine();
		markdownWriter.addString("% " + Support.getDateStamp());
		markdownWriter.newLines(2);
		// Skip the root section if more than one section
		StringBuffer sectionBuffer = new StringBuffer();
		if (binderPathProc.getPathLength() == 1)
		{
			binderPathProc.setPathIndex(0);
			sectionBuffer.append(Support.getBinderName(binderPathProc.getUnmodifiedName()));
		} else
		{
			for (int n = 1; n < binderPathProc.getPathLength(); n++)
			{
				binderPathProc.setPathIndex(n);
				sectionBuffer.append(binderPathProc.getName() + " \\> ");
			}
		}
		String section = sectionBuffer.toString();
		markdownWriter.heading(4, section);
		markdownWriter.newLines(2);
		markdownWriter.separator();
		markdownWriter.newLines(2);

		String autoName = System.getProperty("document.name.auto");
		if ((autoName != null) && (autoName.equals("true")))
		{
			String pageName = Support.getPageName(sourcePath, sourceFileName);
			if (pageName != null)
			{
				int headingLevel = 3;
				String headingLevelString = System.getProperty("document.name.auto.heading.level");
				if ((headingLevelString != null) && (headingLevelString.matches("[1-6]")))
					headingLevel = Integer.parseInt(headingLevelString);
				markdownWriter.heading(headingLevel, pageName);
			}
			markdownWriter.newLines(2);
		}

		String sourceBeforePath = sourceDirectoryPath + "/_resource/auto-generated-file-before.txt";
		markdownWriter.save(sourceBeforePath);
	}

	public void makeFileAfter(String sourceDirectoryPath, String sourcePreviousFileName, String sourceNextFileName,
			BinderPathProc binderPathProc) throws Exception
	{
		MarkdownWriter markdownWriter = new MarkdownWriter();
		markdownWriter.separator();
		// Next and previous pages links
		String webMode = System.getProperty("tree.web.mode");
		String pageName = Support.getPageName(sourceDirectoryPath + "/" + sourceNextFileName, sourceNextFileName);
		if (pageName != null)
		{
			String targetNextFileName;
			if (sourceNextFileName.endsWith("." + Settings.CONTENT_FILE_EXTENSION))
				targetNextFileName = sourceNextFileName.replaceAll("\\." + Settings.CONTENT_FILE_EXTENSION, ".html");
			else
			{
				// Directory
				targetNextFileName = sourceNextFileName + "/index.html";
				pageName += " - Section";
			}
			if ((webMode != null) && (webMode.equals("true")))
				targetNextFileName = targetNextFileName.replaceAll(" ", "_");
			markdownWriter.addString("[ ");
			markdownWriter.link("Next - " + pageName, targetNextFileName);
			markdownWriter.addString(" ] ");
			markdownWriter.newLines(2);
		}
		pageName = Support.getPageName(sourceDirectoryPath + "/" + sourcePreviousFileName, sourcePreviousFileName);
		if (pageName != null)
		{
			String targetPreviousFileName;
			if (sourcePreviousFileName.endsWith("." + Settings.CONTENT_FILE_EXTENSION))
				targetPreviousFileName = sourcePreviousFileName.replaceAll("\\." + Settings.CONTENT_FILE_EXTENSION,
						".html");
			else
			{
				// Directory
				targetPreviousFileName = sourcePreviousFileName + "/index.html";
				pageName += " - Section";
			}
			if ((webMode != null) && (webMode.equals("true")))
				targetPreviousFileName = targetPreviousFileName.replaceAll(" ", "_");
			markdownWriter.addString("[ ");
			markdownWriter.link("Previous - " + pageName, targetPreviousFileName);
			markdownWriter.addString(" ] ");
			markdownWriter.newLines(2);
		}
		// Upper section links    
		int directoryLevel = 0;
		String relativePath = "";
		for (int n = binderPathProc.getPathLength() - 1; n >= 0; n--)
		{
			directoryLevel++;
			String upIndicator = "Up - ";
			if (directoryLevel > 1)
				upIndicator = "Up " + Integer.toString(directoryLevel) + " - ";
			binderPathProc.setPathIndex(n);
			if (n < binderPathProc.getPathLength() - 1)
				relativePath += "../";
			String name = null;
			if (n == 0) // Main index
				name = upIndicator + Support.getBinderName(binderPathProc.getUnmodifiedName()) + " - Main Index";
			else
				name = upIndicator + binderPathProc.getName() + " - Section";
			String urlPath = relativePath + "index.html";
			markdownWriter.addString("[ ");
			markdownWriter.link(name, urlPath);
			markdownWriter.addString(" ] ");
			markdownWriter.newLines(2);
		}

		String sourceAfterPath = sourceDirectoryPath + "/_resource/auto-generated-file-after.txt";
		markdownWriter.save(sourceAfterPath);
	}

	public void makeFilePage(String sourceDirectoryPath, String targetOffsetDirectoryPath, String sourceFileName,
			String sourcePath, String targetPath, String sourcePreviousFileName, String sourceNextFileName,
			BinderPathProc binderPathProc) throws Exception
	{
		// Note many of the arguments can be derived from each other it is left with multiple arguments for efficiency called from convertDirectory 
		Report.debug("Processing: " + sourcePath);
		makeFileBefore(sourceDirectoryPath, sourcePath, sourceFileName, binderPathProc);
		makeFileAfter(sourceDirectoryPath, sourcePreviousFileName, sourceNextFileName, binderPathProc);
		String sourceFileBeforePath = sourceDirectoryPath + "/_resource/auto-generated-file-before.txt";
		String sourceFileAfterPath = sourceDirectoryPath + "/_resource/auto-generated-file-after.txt";
		String[] sourcePathArray =
		{ sourceFileBeforePath, sourcePath, sourceFileAfterPath };
		convertFileFormat(sourcePathArray, targetPath);
	}

	public void makeFilePage(String sourcePath) throws Exception
	{
		if (sourcePath == null)
			throw new BinderException("Error in makeFilePage:\n sourcePath null");
		File sourceFile = new File(sourcePath);
		if (sourceFile.exists() == false)
			throw new BinderException("Error in makeFilePage:\n sourcePath not found");
		if (sourceFile.isFile() == false)
			throw new BinderException("Error in makeFilePage:\n sourcePath not a file");
		String sourceDirectoryPath = sourceFile.getParent();
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in makeFilePage:\n SourceDirectoryPath not found!");
		String[] pathArray = Utilities.splitPath(sourceDirectoryPath);
		//Detect the start of the binder  
		int binderStart = Support.getBinderStartIndexInPath(pathArray);
		String sourceBaseDirectoryPath = Utilities.joinPathArray(pathArray, 0, binderStart - 1);
		String sourceOffsetDirectoryPath = "/" + Utilities.joinPathArray(pathArray, binderStart, pathArray.length - 1);
		String sourceFileName = sourceFile.getName();
		String sourceFileExtension = Settings.CONTENT_FILE_EXTENSION;
		String targetFileExtension = "html";
		String sourceFormat = "markdown";
		setFromFormat(sourceFormat);
		String targetFormat = "html";
		setToFormat(targetFormat);
		String title = System.getProperty("tree.content.title");
		if (((title == null) || (title.toLowerCase().equals("none")) || (title.toLowerCase().equals(""))))
			title = " ";
		if (title.toLowerCase().equals("file"))
			title = sourceFileName.replaceAll("\\." + sourceFileExtension + "$", "." + targetFileExtension);
		setAdditionalArgs("-s" + Settings.ARGS_SEP + "-T" + Settings.ARGS_SEP + title + Settings.ARGS_SEP + "-c"
				+ Settings.ARGS_SEP + Settings.TREE_CONTENT_CSS_NAME + ".css" + Settings.ARGS_SEP);
		String targetBaseDirectoryPath = System.getProperty("tree.target.base.directory.path");
		String targetOffsetDirectoryPath = sourceOffsetDirectoryPath;
		String webMode = System.getProperty("tree.web.mode");
		if ((webMode != null) && (webMode.equals("true")))
			targetOffsetDirectoryPath = targetOffsetDirectoryPath.replaceAll(" ", "_");
		//Test for defined HTML target path
		if ((targetBaseDirectoryPath == null) || (targetBaseDirectoryPath.equals("none")))
			targetBaseDirectoryPath = sourceBaseDirectoryPath;
		String targetDirectoryPath = targetBaseDirectoryPath + targetOffsetDirectoryPath;
		File resourceDirectoryFile = new File(sourceDirectoryPath + "/_resource");
		if (resourceDirectoryFile.exists() == false)
		{
			if (resourceDirectoryFile.mkdirs() == false)
				throw new BinderException("Error in makeFilePage:\n Can't mkdirs resource directory "
						+ resourceDirectoryFile.getPath());
		}
		File targetDirectoryFile = new File(targetDirectoryPath);
		if (targetDirectoryFile.exists() == false)
		{
			if (targetDirectoryFile.mkdirs() == false)
				throw new BinderException("Error in makeFilePage:\n Can't mkdirs target directory "
						+ targetDirectoryPath);
		}
		String targetFileName = sourceFileName.replaceAll("\\." + sourceFileExtension + "$", "." + targetFileExtension);
		if ((webMode != null) && (webMode.equals("true")))
			targetFileName = targetFileName.replaceAll(" ", "_");
		String targetPath = targetDirectoryPath + "/" + targetFileName;
		DirectoryProc directoryProc = new DirectoryProc();
		directoryProc.processFilesOrder(sourceDirectoryPath, sourceFileExtension);
		String orderFilePath = sourceDirectoryPath + "/_resource/order.txt";
		File orderFile = new File(orderFilePath);
		if (orderFile.exists() == false)
			return; // Directory empty
		String orderFileNamesString;
		try
		{
			orderFileNamesString = Utilities.fileToString(orderFilePath);
			orderFileNamesString = orderFileNamesString.trim();
		} catch (IOException ex)
		{
			throw new BinderException("Error in makeFilePage: loading order file\n " + ex.toString());
		}
		BinderPathProc binderPathProc = new BinderPathProc(sourceBaseDirectoryPath, sourceOffsetDirectoryPath);
		String[] orderFileNames = orderFileNamesString.split("\n"); // Note - line separator does not seem to work with split
		String sourcePreviousFileName = null;
		String sourceNextFileName = null;
		for (int n = 0; n < orderFileNames.length; n++)
		{
			if (sourceFileName.equals(orderFileNames[n]))
			{
				for (int i = n - 1; i >= 0; i--)
				{
					if (orderFileNames[i].toLowerCase().endsWith(Settings.INSERT_FILE_SUFFIX) == false)
					{
						sourcePreviousFileName = orderFileNames[i];
						break;
					}
				}
				for (int i = n + 1; i < orderFileNames.length; i++)
				{
					if (orderFileNames[i].toLowerCase().endsWith(Settings.INSERT_FILE_SUFFIX) == false)
					{
						sourceNextFileName = orderFileNames[i];
						break;
					}
				}
			}
		}
		makeFilePage(sourceDirectoryPath, targetOffsetDirectoryPath, sourceFileName, sourcePath, targetPath,
				sourcePreviousFileName, sourceNextFileName, binderPathProc);
		directoryProc.deleteDirectoryAutoGeneratedFiles(sourceDirectoryPath);
	}

}
