package processing;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import common.BinderException;
import common.Support;
import common.Utilities;

import config.Settings;
import formats.EmptyFileWriter;
import formats.MarkdownWriter;

public class DirectoryProc
{

	public void cleanDirectory(String directoryPath, String excludedFileExtensions) throws Exception
	{
		File directoryFile = new File(directoryPath);
		if (directoryFile.exists() == false)
		{
			throw new BinderException("Error in cleanDirectory: directory does not exist");
		}
		//      intactFilesString is a string of files not to be deleted separated by commas        
		if (excludedFileExtensions == null)
		{
			throw new BinderException("Error in cleanDirectory: excludedFilesString null");
		}
		String[] excludedFileExtensionsArray = excludedFileExtensions.split(",");
		File[] files = directoryFile.listFiles();
		for (int n = 0; n < files.length; n++)
		{
			if (files[n].isDirectory())
				continue;
			boolean exclude = false;
			for (int m = 0; m < excludedFileExtensionsArray.length; m++)
			{
				String tempFileName = files[n].getName().toLowerCase();
				if ((tempFileName.endsWith(excludedFileExtensionsArray[m])) || (tempFileName.startsWith("_")))
					exclude = true;
			}
			if (exclude == false)
			{
				if (files[n].delete() == false)
					continue;
			}
		}
	}

	public void deleteDirectoryAutoGeneratedFiles(String directoryPath)
	{
		if (directoryPath == null)
			return;
		String resourceDirectoryPath = directoryPath + "/_resource";
		File resourceDirectoryFile = new File(resourceDirectoryPath);
		if (resourceDirectoryFile.exists() == false)
			return; // resource directory might not exists after binder creation 	
		String[] resourceFileNames = resourceDirectoryFile.list();
		for (int n = 0; n < resourceFileNames.length; n++)
		{
			if (resourceFileNames[n].startsWith("auto-generated"))
			{
				File resourceFile = new File(resourceDirectoryPath + "/" + resourceFileNames[n]);
				if (resourceFile.delete() == false)
					continue;
			}
		}
	}

	public void processFilesOrder(String sourceDirectoryPath, String sourceFileExtension) throws Exception
	{
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
		{
			throw new BinderException("Error in processFilesOrder: directory does not exist");
		}
		if (sourceFileExtension == null)
		{
			throw new BinderException("Error in processFilesOrder: fileExtension null");
		}
		String ls = System.getProperty("line.separator");
		String orderFilePath = sourceDirectoryPath + "/_resource/order.txt";
		File orderFile = new File(orderFilePath);
		String[] directoryFileNames = sourceDirectoryFile.list();
		if (orderFile.exists() == false)
		{
			StringBuffer directoryFileNamesBuffer = new StringBuffer();
			for (int n = 0; n < directoryFileNames.length; n++)
			{
				if ((directoryFileNames[n].startsWith("_")) || (directoryFileNames[n].startsWith(".")))
					continue;
				String sourcePath = sourceDirectoryPath + "/" + directoryFileNames[n];
				if (new File(sourcePath).isDirectory())
				{
					// Only include source directories
					sourcePath = sourceDirectoryPath + "/" + directoryFileNames[n] + "/_resource";
					if (new File(sourcePath).exists())
						directoryFileNamesBuffer.append(directoryFileNames[n] + ls);
				} else if (directoryFileNames[n].toLowerCase().endsWith(sourceFileExtension))
					directoryFileNamesBuffer.append(directoryFileNames[n] + ls);
			}
			String directoryFileNamesString = directoryFileNamesBuffer.toString().trim();
			try
			{
				if (directoryFileNamesString.equals("") == false)
					Utilities.stringToFile(orderFilePath, directoryFileNamesString);
			} catch (IOException ex)
			{
				throw new BinderException("Error in processFilesOrder: " + ex.toString());
			}
			return;
		}
		String orderFileNamesString = "";
		try
		{
			orderFileNamesString = Utilities.fileToString(orderFilePath);
			orderFileNamesString = orderFileNamesString.trim();
		} catch (IOException ex)
		{
			throw new BinderException("Error in processFilesOrder: " + ex.toString());
		}
		List<String> directoryFileNamesList = new ArrayList<String>();
		// Filter non relevant file names
		for (int n = 0; n < directoryFileNames.length; n++)
		{
			if ((directoryFileNames[n].startsWith("_")) || (directoryFileNames[n].startsWith(".")))
				continue;
			String sourcePath = sourceDirectoryPath + "/" + directoryFileNames[n];
			if (new File(sourcePath).isDirectory())
			{
				sourcePath = sourceDirectoryPath + "/" + directoryFileNames[n] + "/_resource";
				if (new File(sourcePath).exists())
					directoryFileNamesList.add(directoryFileNames[n]);
			} else if (directoryFileNames[n].toLowerCase().endsWith(sourceFileExtension))
				directoryFileNamesList.add(directoryFileNames[n]);
		}
		String[] orderFileNames = orderFileNamesString.split("\n"); // Note line separator does not seem to work     
		List<String> orderFileNamesList = new ArrayList<String>();
		for (int n = 0; n < orderFileNames.length; n++)
		{
			orderFileNames[n] = orderFileNames[n].trim();
			// Add property file if indicated by the user by marking it with an asterisc suffix
			if (orderFileNames[n].endsWith("*"))
			{
				orderFileNames[n] = orderFileNames[n].replaceAll("\\*$", "");
				orderFileNames[n] = orderFileNames[n].trim();
				Settings.setFileDirProperties(sourceDirectoryPath, orderFileNames[n]);
			}
			if (directoryFileNamesList.contains(orderFileNames[n]))
				orderFileNamesList.add(orderFileNames[n]);
		}
		// Add newly added files to the order list
		for (int n = 0; n < directoryFileNamesList.size(); n++)
		{
			if (orderFileNamesList.contains(directoryFileNamesList.get(n)) == false)
				orderFileNamesList.add(directoryFileNamesList.get(n));
		}
		String previousOrderFileNamesString = orderFileNamesString;
		StringBuffer orderFileNamesBuffer = new StringBuffer();
		for (int n = 0; n < orderFileNamesList.size(); n++)
		{
			orderFileNamesBuffer.append(orderFileNamesList.get(n) + ls);
		}
		orderFileNamesString = orderFileNamesBuffer.toString().trim();
		if (orderFileNamesString.equals(previousOrderFileNamesString))
			return; // No need to save
		try
		{
			Utilities.stringToFile(orderFilePath, orderFileNamesString);
		} catch (IOException ex)
		{
			throw new BinderException("Error in processFilesOrder: " + ex.toString());
		}
	}

	public void deleteRedundantTargetDirectoryFiles(String sourceDirectoryPath, String targetDirectoryPath,
			String sourceFileExtension, String targetFileExtension)
	{
		File targetDirectoryFile = new File(targetDirectoryPath);
		String[] targetFileNames = targetDirectoryFile.list();
		for (int n = 0; n < targetFileNames.length; n++)
		{
			if (targetFileNames[n].toLowerCase().endsWith(".css"))
				continue;
			if (targetFileNames[n].toLowerCase().endsWith("." + targetFileExtension))
			{
				//  Do not delete exceptions
				if (targetFileNames[n].startsWith("index"))
					continue;
				String sourceFileName = targetFileNames[n].replaceAll("\\." + targetFileExtension + "$", "."
						+ sourceFileExtension);
				String sourcePath = sourceDirectoryPath + "/" + sourceFileName;
				// Check for both source with underlines and spaces    
				boolean sourceDiscovered = false;
				File sourceFile = new File(sourcePath);
				if (sourceFile.exists())
					sourceDiscovered = true;
				else
				// Try to remove underlines from file name  
				{
					sourceFileName = sourceFileName.replaceAll("_", " ");
					sourcePath = sourceDirectoryPath + "/" + sourceFileName;
					sourceFile = new File(sourcePath);
					if (sourceFile.exists())
						sourceDiscovered = true;
				}
				if (sourceDiscovered == false)
				{
					String targetPath = targetDirectoryPath + "/" + targetFileNames[n];
					File targetFile = new File(targetPath);
					if (targetFile.exists())
					{
						if (targetFile.delete() == false)
							continue;
					}
				}
			}
		}
	}

	public void convertDirectory(String sourceDirectoryPath, String targetDirectoryPath,
			String targetOffsetDirectoryPath, String targetFileExtension, String fromFormat, String toFormat,
			BinderPathProc binderPathProc) throws Exception
	{
		String orderFilePath = sourceDirectoryPath + "/_resource/order.txt";
		File orderFile = new File(orderFilePath);
		if (orderFile.exists() == false)
			return; // Directory empty
		String orderFileNamesString;
		try
		{
			orderFileNamesString = Utilities.fileToString(orderFilePath);
			orderFileNamesString = orderFileNamesString.trim();
		} catch (IOException ex)
		{
			throw new BinderException("Error in convertDirectory: " + ex.toString());
		}
		ContentProc contentProc = new ContentProc();
		contentProc.setFromFormat(fromFormat);
		contentProc.setToFormat(toFormat);
		String[] orderFileNames = orderFileNamesString.split("\n"); // Note - line separator does not seem to work with split    
		for (int n = 0; n < orderFileNames.length; n++)
		{
			String sourceFileName = orderFileNames[n];
			if (sourceFileName.toLowerCase().endsWith(Settings.INSERT_FILE_SUFFIX))
				continue;
			String targetFileName = sourceFileName.replaceAll("\\.[a-zA-Z0-9]{2,4}$", "." + targetFileExtension);
			String webMode = System.getProperty("tree.web.mode");
			if ((webMode != null) && (webMode.equals("true")))
				targetFileName = targetFileName.replaceAll(" ", "_");
			String sourcePath = sourceDirectoryPath + "/" + sourceFileName;
			String targetPath = targetDirectoryPath + "/" + targetFileName;
			File targetFile = new File(targetPath);
			if ((targetFile.exists()) && (targetFile.isDirectory()))
				continue;
			File sourceFile = new File(sourcePath);
			if ((targetFile.exists() == false) || ((targetFile.lastModified()) < (sourceFile.lastModified())))
			{
				String sourcePreviousFileName = null;
				for (int i = n - 1; i >= 0; i--)
				{
					if (orderFileNames[i].toLowerCase().endsWith(Settings.INSERT_FILE_SUFFIX) == false)
					{
						sourcePreviousFileName = orderFileNames[i];
						break;
					}
				}
				String sourceNextFileName = null;
				for (int i = n + 1; i < orderFileNames.length; i++)
				{
					if (orderFileNames[i].toLowerCase().endsWith(Settings.INSERT_FILE_SUFFIX) == false)
					{
						sourceNextFileName = orderFileNames[i];
						break;
					}
				}
				String title = System.getProperty("tree.content.title");
				if (((title == null) || (title.toLowerCase().equals("none")) || (title.toLowerCase().equals(""))))
					title = " ";
				if (title.toLowerCase().equals("file"))
					title = targetFileName;
				contentProc.setAdditionalArgs("-s" + Settings.ARGS_SEP + "-T" + Settings.ARGS_SEP + title
						+ Settings.ARGS_SEP + "-c" + Settings.ARGS_SEP + Settings.TREE_CONTENT_CSS_NAME + ".css"
						+ Settings.ARGS_SEP);
				contentProc.makeFilePage(sourceDirectoryPath, targetOffsetDirectoryPath, sourceFileName, sourcePath,
						targetPath, sourcePreviousFileName, sourceNextFileName, binderPathProc);
			}
		}
	}

	public void makeRequiredStyleSheets(String sourceDirectoryPath, String targetDirectoryPath) throws Exception
	{
		String sourceBinderDirectoryPath = System.getProperty("source.binder.directory.path");
		if (sourceBinderDirectoryPath.equals(sourceDirectoryPath))
			makeStyleSheet(sourceDirectoryPath, targetDirectoryPath, Settings.TREE_FRONT_CSS_NAME);
		else
			makeStyleSheet(sourceDirectoryPath, targetDirectoryPath, Settings.TREE_SECTION_CSS_NAME);
		String orderFilePath = sourceDirectoryPath + "/_resource/order.txt";
		File orderFile = new File(orderFilePath);
		if (orderFile.exists() == false)
			return; // Most probably directory emptty  
		String orderFileNamesString;
		try
		{
			orderFileNamesString = Utilities.fileToString(orderFilePath);
			orderFileNamesString = orderFileNamesString.trim();
		} catch (IOException ex)
		{
			throw new BinderException("Error in makeRequiredStyleSheets: " + ex.toString());
		}
		String[] orderFileNames = orderFileNamesString.split("\n"); // Note - line separator does not seem to work with split    
		for (int n = 0; n < orderFileNames.length; n++)
		{
			File contentFile = new File(sourceDirectoryPath + "/" + orderFileNames[n]);
			if (contentFile.isFile())
			{
				makeStyleSheet(sourceDirectoryPath, targetDirectoryPath, Settings.TREE_CONTENT_CSS_NAME);
				break;
			}
		}
	}

	public void makeStyleSheet(String sourceDirectoryPath, String targetDirectoryPath, String templateName)
			throws Exception
	{
		String templatePath = Support.getStyleSheetTemplatePath(sourceDirectoryPath, templateName);
		File templateFile = new File(templatePath);
		String cssPath = targetDirectoryPath + "/" + templateName + ".css";
		File cssFile = new File(cssPath);
		// Do not overide unless there is a change
		if ((cssFile.exists() == false) || ((cssFile.lastModified()) < (templateFile.lastModified())))
			Utilities.copyFile(templateFile, cssFile);
	}

	public void makePageIndex(String sourceDirectoryPath, String targetDirectoryPath, BinderPathProc binderPathProc)
			throws Exception
	{
		String orderFilePath = sourceDirectoryPath + "/_resource/order.txt";
		File orderFile = new File(orderFilePath);
		if (orderFile.exists() == false)
			return; // Directory empty
		MarkdownWriter markdownWriter = new MarkdownWriter();
		// Generate Pandoc header
		markdownWriter.addString("%");
		markdownWriter.newLine();
		String author = System.getProperty("binder.author");
		if ((author == null) || (author.equals("none")))
			author = "";
		markdownWriter.addString("% " + author);
		markdownWriter.newLine();
		markdownWriter.addString("% " + Support.getDateStamp());
		markdownWriter.newLines(2);
		StringBuffer sectionBuffer = new StringBuffer();
		if (binderPathProc.getPathLength() == 1)
		{
			binderPathProc.setPathIndex(0);
			sectionBuffer.append(Support.getBinderName(binderPathProc.getUnmodifiedName()));
			markdownWriter.heading(3, sectionBuffer.toString());
			markdownWriter.newLines(2);
		} else
		{
			//          Skip the root section (the binder itself) if more than one section 
			for (int n = 1; n < binderPathProc.getPathLength(); n++)
			{
				binderPathProc.setPathIndex(n);
				sectionBuffer.append(binderPathProc.getName() + " \\> ");
			}
			markdownWriter.heading(3, sectionBuffer.toString());
			markdownWriter.newLines(2);
		}
		// Links to sections and files
		String orderFileNamesString;
		try
		{
			orderFileNamesString = Utilities.fileToString(orderFilePath);
			orderFileNamesString = orderFileNamesString.trim();
		} catch (IOException ex)
		{
			throw new BinderException("Error in makePageIndex: loading order file\n " + ex.toString());
		}
		String[] orderFileNames = orderFileNamesString.split("\n"); // Note - line separator does not seem to work with split
		for (int n = 0; n < orderFileNames.length; n++)
		{
			// If insert file first omit the line separator  
			if ((n == 0) && (orderFileNames[0].toLowerCase().endsWith(Settings.INSERT_FILE_SUFFIX) == false))
			{
				markdownWriter.separator();
				markdownWriter.newLines(2);
			}
			String fileDirName = orderFileNames[n].replaceAll("\\.[a-zA-Z0-9]{2,4}$", "");
			String urlPath = null;
			String sourcePath = sourceDirectoryPath + "/" + orderFileNames[n];
			boolean directoryFlag = false;
			File sourceFile = new File(sourcePath);
			if (sourceFile.isDirectory())
			{
				// Check that the directory is not empty 
				String subDirOrderFilePath = sourceDirectoryPath + "/" + fileDirName + "/_resource/order.txt";
				File subDirOrderFile = new File(subDirOrderFilePath);
				if (subDirOrderFile.exists() == false)
					continue; // Directory empty
				urlPath = "./" + fileDirName + "/index.html";
				directoryFlag = true;
			} else
			// File
			{
				if (sourcePath.toLowerCase().endsWith(Settings.INSERT_FILE_SUFFIX))
				{
					String insert = Utilities.fileToString(sourcePath);
					markdownWriter.addString(insert);
					continue;
				}
				urlPath = fileDirName + ".html";
			}
			String webMode = System.getProperty("tree.web.mode");
			if ((webMode != null) && (webMode.equals("true")))
				urlPath = urlPath.replaceAll(" ", "_");
			String name = Support.getPageName(sourcePath, fileDirName);
			if (directoryFlag)
				name += " - Section";
			markdownWriter.link(name, urlPath);
			markdownWriter.newLines(2);
		}
		if (binderPathProc.getPathLength() > 1)
			markdownWriter.separator();
		// Upper section links    
		int directoryLevel = 0;
		String relativePath = "";
		for (int n = binderPathProc.getPathLength() - 2; n >= 0; n--)
		{
			directoryLevel++;
			String upIndicator = "Up - ";
			if (directoryLevel > 1)
				upIndicator = "Up " + Integer.toString(directoryLevel) + " - ";
			binderPathProc.setPathIndex(n);
			relativePath += "../";
			String name = null;
			if (n == 0) // Main index
				name = upIndicator + Support.getBinderName(binderPathProc.getUnmodifiedName()) + " - Main Index";
			else
				name = upIndicator + binderPathProc.getName() + " - Section";
			String urlPath = relativePath + "index.html";
			markdownWriter.addString("[ ");
			markdownWriter.link(name, urlPath);
			markdownWriter.addString(" ] ");
			markdownWriter.newLines(2);
		}

		String sourcePath = sourceDirectoryPath + "/_resource/auto-generated-index.txt";
		markdownWriter.save(sourcePath);
		String targetPath = targetDirectoryPath + "/index.html";
		ContentProc contentProc = new ContentProc();
		contentProc.setFromFormat("markdown");
		contentProc.setToFormat("html");
		if (binderPathProc.getPathLength() == 1) // Main index
			contentProc.setAdditionalArgs("-s" + Settings.ARGS_SEP + "-T" + Settings.ARGS_SEP
					+ System.getProperty("tree.front.title") + Settings.ARGS_SEP + "-c" + Settings.ARGS_SEP
					+ Settings.TREE_FRONT_CSS_NAME + ".css" + Settings.ARGS_SEP);
		else
			contentProc.setAdditionalArgs("-s" + Settings.ARGS_SEP + "-T" + Settings.ARGS_SEP
					+ System.getProperty("tree.section.title") + Settings.ARGS_SEP + "-c" + Settings.ARGS_SEP
					+ Settings.TREE_SECTION_CSS_NAME + ".css" + Settings.ARGS_SEP);
		String[] sourcePathArray =
		{ sourcePath };
		contentProc.convertFileFormat(sourcePathArray, targetPath);
	}

	private void processBinderDirectory(String sourceBaseDirectoryPath, String sourceOffsetDirectoryPath)
			throws Exception
	{
		String sourceFileExtension = Settings.CONTENT_FILE_EXTENSION;
		String targetFileExtension = "html";
		String sourceFormat = "markdown";
		String targetFormat = "html";
		String sourceDirectoryPath = sourceBaseDirectoryPath + sourceOffsetDirectoryPath;
		String targetBaseDirectoryPath = System.getProperty("tree.target.base.directory.path");
		String targetOffsetDirectoryPath = sourceOffsetDirectoryPath;
		String webMode = System.getProperty("tree.web.mode");
		if ((webMode != null) && (webMode.equals("true")))
		{
			targetOffsetDirectoryPath = targetOffsetDirectoryPath.replaceAll(" ", "_");
			if (targetBaseDirectoryPath.endsWith(Settings.WEB_BINDER_INDICATOR) == false) // Mark the target directory
			{
				targetOffsetDirectoryPath = targetOffsetDirectoryPath.replaceAll("_$", Settings.WEB_BINDER_INDICATOR);
				targetOffsetDirectoryPath = targetOffsetDirectoryPath.replaceAll("_\\/", Settings.WEB_BINDER_INDICATOR
						+ "/");
			}
		}
		// Test for defined HTML target path
		if ((targetBaseDirectoryPath == null) || (targetBaseDirectoryPath.equals("none")))
			targetBaseDirectoryPath = sourceBaseDirectoryPath;
		String targetDirectoryPath = targetBaseDirectoryPath + targetOffsetDirectoryPath;
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in processBinderDirectory:\n Source Directory not found!");
		String resourceDirectoryPath = sourceDirectoryPath + "/_resource";
		File resourceDirectoryFile = new File(resourceDirectoryPath);
		if (resourceDirectoryFile.exists() == false)
		{
			if (resourceDirectoryFile.mkdirs() == false)
				throw new BinderException("Error in processBinderDirectory:\n + Can't mkdirs resourceDirectory "
						+ resourceDirectoryPath);
		}
		File targetDirectoryFile = new File(targetDirectoryPath);
		if (targetDirectoryFile.exists() == false)
		{
			if (targetDirectoryFile.mkdirs() == false)
				throw new BinderException("Error in processBinderDirectory:\n Can't mkdirs targetDirectory "
						+ targetDirectoryPath);
		}
		EmptyFileWriter.createEmptyMarkdownFile(resourceDirectoryPath);
		processFilesOrder(sourceDirectoryPath, sourceFileExtension);
		deleteRedundantTargetDirectoryFiles(sourceDirectoryPath, targetDirectoryPath, sourceFileExtension,
				targetFileExtension);
		makeRequiredStyleSheets(sourceDirectoryPath, targetDirectoryPath);
		BinderPathProc binderPathProc = new BinderPathProc(sourceBaseDirectoryPath, sourceOffsetDirectoryPath);
		makePageIndex(sourceDirectoryPath, targetDirectoryPath, binderPathProc);
		convertDirectory(sourceDirectoryPath, targetDirectoryPath, targetOffsetDirectoryPath, targetFileExtension,
				sourceFormat, targetFormat, binderPathProc);
		// Leave binder with content and resource files only
		deleteDirectoryAutoGeneratedFiles(sourceDirectoryPath);
		if ((webMode != null) && (webMode.equals("true")))
		{
			cleanDirectory(sourceDirectoryPath, Settings.FILES_EXCLUDED_FROM_CLEANING);
			postProcessWebDirectory(targetDirectoryPath);
		}
	}

	public void processBinderDirectory(String sourceDirectoryPath) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in processBinderDirectory:\n SourceDirectoryPath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in processBinderDirectory:\n SourceDirectoryPath not found!");
		if (sourceDirectoryFile.isDirectory() == false)
			throw new BinderException("Error in processBinderDirectory:\n sourcePath not a directory");
		String[] pathArray = Utilities.splitPath(sourceDirectoryPath);
		//  Detect the start of the binder  
		int binderStart = Support.getBinderStartIndexInPath(pathArray);
		String sourceBaseDirectoryPath = Utilities.joinPathArray(pathArray, 0, binderStart - 1);
		String sourceOffsetDirectoryPath = "/" + Utilities.joinPathArray(pathArray, binderStart, pathArray.length - 1);
		processBinderDirectory(sourceBaseDirectoryPath, sourceOffsetDirectoryPath);
	}

	public void postProcessWebDirectory(String directoryPath) throws Exception
	{
		HTMLPostProc htmlPostProc = new HTMLPostProc();
		File directoryFile = new File(directoryPath);
		if (directoryFile.exists() == false)
		{
			throw new BinderException("Error in postProcessWebDirectory: directory does not exist");
		}
		File[] files = directoryFile.listFiles();
		for (int n = 0; n < files.length; n++)
		{
			if (files[n].isDirectory())
				continue;
			String filePath = files[n].getPath();
			if (filePath.toLowerCase().endsWith(".html"))
				htmlPostProc.processFile(filePath);
			;
		}
	}

}
