package processing;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import common.BinderException;
import config.Settings;

import formats.FileDirProperties;

public class FileDirPropertiesProc
{
	private static FileDirPropertiesProc fFileDirPropertiesProc;
	private Map<String, String> fAliasProperties;

	private FileDirPropertiesProc()
	{
		fAliasProperties = new HashMap<String, String>();
	}

	public synchronized static FileDirPropertiesProc getInstance()
	{
		if (fFileDirPropertiesProc == null)
			fFileDirPropertiesProc = new FileDirPropertiesProc();
		return (fFileDirPropertiesProc);
	}

	public String getAliasProperty(String key)
	{
		if (key == null)
			return (null);
		key = key.replaceAll("\\\\", "/");
		return (fAliasProperties.get(key));
	}

	public Map<String, String> getFileDirProperties(String propertiesPath) throws Exception
	{
		File propertiesFile = new File(propertiesPath);
		if (propertiesFile.exists() == false)
			return (null); // Properties not defined  
		FileDirProperties fileDirProperties = new FileDirProperties();
		try
		{
			fileDirProperties.load(propertiesPath);
		} catch (Exception ex)
		{
			throw new BinderException(
					"Error in loadFileDirProperties:\n properties file unable to load properties file " + ex.toString());
		}
		fileDirProperties.propertiesToMap();
		return (fileDirProperties.getProperties());
	}

	public void loadFileDirProperties(String directoryPath) throws Exception
	{
		String propertiesDirectoryPath = directoryPath + "/_resource/_properties";
		File propertiesDirectoryFile = new File(propertiesDirectoryPath);
		if (propertiesDirectoryFile.exists() == false)
		{
			// ignore not all directories have properties associated with them
			return;
		}
		String[] children = propertiesDirectoryFile.list();
		for (int n = 0; n < children.length; n++)
		{
			if ((children[n].startsWith(".")) || (children[n].equals(Settings.BINDER_PROPERTIES_FILE_NAME + ".txt")))
				continue;
			Map<String, String> properties = getFileDirProperties(propertiesDirectoryPath + "/" + children[n]);
			if (properties == null)
				continue;
			String alias = properties.get("alias");
			if ((alias != null) && (alias.equals("none") == false))
			{
				String fileDirPath;
				if (children[n].equals(Settings.DIRECTORY_PROPERTIES_FILE_NAME + ".txt"))
					fileDirPath = directoryPath;
				else
				{
					String fileDirName = children[n].replaceAll("-properties.txt$", "");
					fileDirPath = directoryPath + "/" + fileDirName;
				}
				File fileDirFile = new File(fileDirPath);
				if (fileDirFile.exists() == false)
					continue;
				// Harmonize path separators to ensure match
				fileDirPath = fileDirPath.replaceAll("\\\\", "/");
				fAliasProperties.put(fileDirPath, alias);
			}
		}
	}

	private void loadFileDirPropertiesPartialBinderTreeRecursive(String sourceDirectoryPath) throws Exception
	{
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		File[] childrenFiles = sourceDirectoryFile.listFiles();
		for (int n = 0; n < childrenFiles.length; n++)
		{
			if (childrenFiles[n].isDirectory())
			{
				String name = childrenFiles[n].getName();
				if ((name.startsWith("_")) || (name.startsWith(".")))
					continue;
				String childDirectoryPath = childrenFiles[n].getPath();
				loadFileDirPropertiesPartialBinderTreeRecursive(childDirectoryPath);
				loadFileDirProperties(childDirectoryPath);
			}
		}
	}

	private void loadFileDirPropertiesPartialBinderTree(String sourceDirectoryPath) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in loadFileDirPropertiesPartialBinderTree:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in loadFileDirPropertiesPartialBinderTree:\n Source Directory not found!");
		if (sourceDirectoryFile.isDirectory() == false)
			throw new BinderException("Error in loadFileDirPropertiesPartialBinderTree:\n sourcePath not a directory");
		loadFileDirPropertiesPartialBinderTreeRecursive(sourceDirectoryPath);
		loadFileDirProperties(sourceDirectoryPath);
	}

	public void loadFileDirPropertiesAllBinderTree(String sourceDirectoryPath) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in loadFileDirPropertiesAllBinderTree:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in loadFileDirPropertiesAllBinderTree:\n Source Directory not found!");
		if (sourceDirectoryFile.isDirectory() == false)
			throw new BinderException("Error in loadFileDirPropertiesAllBinderTree:\n sourcePath not a directory");
		String sourceBinderDirectoryPath = System.getProperty("source.binder.directory.path");
		loadFileDirPropertiesPartialBinderTree(sourceBinderDirectoryPath);
	}

}
