package processing;

import common.Report;
import common.Utilities;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

public class MarkdownPandocPostProcn
{
	private boolean fFileModified = false;

	public void processFile(String filePath) throws Exception
	{
		String fileString = Utilities.fileToString(filePath);
		fileString = processString(fileString);
		if (fFileModified)
		{
			Report.debug("Post processing Markdown Pandoc: " + filePath);
			Utilities.stringToFile(filePath, fileString);
		}
	}

	private String processString(String fileString) throws Exception
	{
		String lines[] = fileString.split("\n");
		boolean codeDetected = false;
		for (int n = 0; n < lines.length; n++)
		{
			if (codeDetected)
			{
				if (lines[n].trim().length() > 0)
				{
					lines[n] = lines[n].replaceAll("^[\\s]+", "");
					if (n == lines.length - 1)
					{
						lines[n] += System.getProperty("line.separator") + "~~~" + System.getProperty("line.separator")
								+ System.getProperty("line.separator");
						codeDetected = false;
					}
					continue;
				} // Empty line
				lines[n] = "~~~";
				if ((n == lines.length - 1) || (lines[n + 1].trim().length() > 0))
					lines[n] += System.getProperty("line.separator");
				codeDetected = false;
				continue;
			} else
			{
				if (n > 0)
				{
					if (lines[n].matches("^[\\s]+[a-zA-Z0-9].+?"))
					{
						int previousLeadingSpaces = countLeadingSpaces(lines[n - 1]);
						int leadingSpaces = countLeadingSpaces(lines[n]);
						if (leadingSpaces - 4 >= previousLeadingSpaces)
						{
							lines[n] = lines[n].replaceAll("^[\\s]+", "");
							lines[n] = "~~~" + System.getProperty("line.separator") + lines[n];
							if (lines[n - 1].trim().length() > 0)
								lines[n] = System.getProperty("line.separator") + lines[n];
							codeDetected = true;
							fFileModified = true;
							continue;
						}
					}
				}
			}
			if (lines[n].endsWith("  "))
			{
				lines[n] = lines[n].replaceAll("  $", "\\\\");
				fFileModified = true;
			}
		}
		fileString = Utilities.joinArray(lines, 0, lines.length - 1, System.getProperty("line.separator"));
		return (fileString);
	}

	private int countLeadingSpaces(String string)
	{
		for (int n = 0; n < string.length(); n++)
		{
			if (string.charAt(n) != ' ')
				return (n + 1);
		}
		return (0);
	}

}
