package processing;

//Copyright Isaac Porat (c) 2010.  All rights reserved.  License Lesser GPL see accompanying file.

import java.io.File;

import common.BinderException;
import common.Report;
import common.Support;
import common.Utilities;

import config.Settings;
import formats.CSSWriter;

public class TreeProc extends DirectoryProc
{
	private DirectoryProc fDirectoryProc;

	public TreeProc()
	{
		fDirectoryProc = new DirectoryProc();
	}

	public String initializeBinderTree(String sourceDirectoryPath) throws Exception
	{
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error, processing directory does not exist:\n" + sourceDirectoryPath);
		String sourceBinderDirectoryPath = getSourceBinderDirectoryPath(sourceDirectoryPath);
		System.setProperty("source.binder.directory.path", sourceBinderDirectoryPath);
		String binderName = getBinderName(sourceDirectoryPath);
		Settings.setBinderProperties(sourceBinderDirectoryPath, binderName);
		Report.debug(System.getProperty("debug.operation.mode"));
		Report.debug(System.getProperty("debug.source"));
		generateExternalStyleTemplates(sourceBinderDirectoryPath, Settings.TREE_FRONT_CSS_NAME);
		generateExternalStyleTemplates(sourceBinderDirectoryPath, Settings.TREE_SECTION_CSS_NAME);
		generateExternalStyleTemplates(sourceBinderDirectoryPath, Settings.TREE_CONTENT_CSS_NAME);
		generateInternalStyleTemplate(sourceBinderDirectoryPath, Settings.DOCUMENT_INTERNAL_CSS_NAME);
				return (sourceBinderDirectoryPath);
	}

	private void generateExternalStyleTemplates(String sourceBinderDirectoryPath, String templateName) throws Exception
	{
		/*
		 * Generate the external Style Sheets template file
		 * Do not replace Style Sheet template file as they could have been modified by the user
		 */
		String templatePath = sourceBinderDirectoryPath + "/_resource/" + templateName + "-template.txt";
		File templateFile = new File(templatePath);
		if (templateFile.exists() == false)
		{
			String caption = templateName.replaceAll("tree-", "");
			caption = "Tree " + caption.substring(0, 1).toUpperCase() + caption.substring(1).toLowerCase();
			(new CSSWriter()).makeDefaultExternal(templatePath, caption);
		}
	}

	private void generateInternalStyleTemplate(String sourceBinderDirectoryPath, String templateName) throws Exception
	{
		/*
		 * Generate the internal Style Sheets template file
		 */
		String templatePath = sourceBinderDirectoryPath + "/_resource/" + templateName + "-template.txt";
		File templateFile = new File(templatePath);
		if (templateFile.exists() == false)
		{
			String caption = "Document";
			(new CSSWriter()).makeDefaultInternal(templatePath, caption);
		}
	}

	public String getBinderName(String sourceDirectoryPath) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in getBinderName:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in getBinderName:\n Source Directory not found!");
		String[] pathArray = Utilities.splitPath(sourceDirectoryPath);
		//      Detect the start of the binder  
		int binderStart = Support.getBinderStartIndexInPath(pathArray);
		String name = pathArray[binderStart].replaceAll("_", " ").trim();
		return (name);
	}

	public String getSourceBinderDirectoryPath(String sourceDirectoryPath) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in getSourceBinderDirectoryPath:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in getSourceBinderDirectoryPath:\n Source Directory not found!");
		String[] pathArray = Utilities.splitPath(sourceDirectoryPath);
		//      Detect the start of the binder  
		int binderStart = Support.getBinderStartIndexInPath(pathArray);
		String sourceBinderDirectoryPath = Utilities.joinPathArray(pathArray, 0, binderStart);
		return (sourceBinderDirectoryPath);
	}

	public String getTargetBaseWebDirectoryPath(String sourceDirectoryPath) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in getTargetBaseWebDirectoryPath:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in getTargetBaseWebDirectoryPath:\n Source Directory not found!");
		String[] pathArray = Utilities.splitPath(sourceDirectoryPath);
		//      Detect the start of the binder  
		int binderStart = Support.getBinderStartIndexInPath(pathArray);
		// path depends if base is a container binder
		boolean containerBinder = false;
		if (pathArray[binderStart - 1].endsWith("_"))
		{
			containerBinder = true;
			pathArray[binderStart - 1] = pathArray[binderStart - 1].replaceAll(" ", "_");
			pathArray[binderStart - 1] = pathArray[binderStart - 1].replaceAll("_$", "");
		}
		String sourceBaseDirectoryPath = Utilities.joinPathArray(pathArray, 0, binderStart - 1);
		if (containerBinder)
			sourceBaseDirectoryPath += Settings.WEB_BINDER_INDICATOR;
		return (sourceBaseDirectoryPath);
	}

	public String getTargetBinderWebDirectoryPath(String sourceDirectoryPath) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in getTargetBinderWebDirectoryPath:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in getTargetBinderWebDirectoryPath:\n Source Directory not found!");
		String[] pathArray = Utilities.splitPath(sourceDirectoryPath);
		//      Detect the start of the binder  
		int binderStart = Support.getBinderStartIndexInPath(pathArray);
		//   path depends if base is a container binder
		boolean containerBinder = false;
		if (pathArray[binderStart - 1].endsWith("_"))
		{
			containerBinder = true;
			pathArray[binderStart - 1] = pathArray[binderStart - 1].replaceAll(" ", "_");
			pathArray[binderStart - 1] = pathArray[binderStart - 1].replaceAll("_$", "");
			pathArray[binderStart - 1] += Settings.WEB_BINDER_INDICATOR;
		}
		pathArray[binderStart] = pathArray[binderStart].replaceAll(" ", "_");
		if (containerBinder == false)
		{
			pathArray[binderStart] = pathArray[binderStart].replaceAll("_$", "");
			pathArray[binderStart] += Settings.WEB_BINDER_INDICATOR;
		}
		String targetBinderDirectoryPath = Utilities.joinPathArray(pathArray, 0, binderStart);
		return (targetBinderDirectoryPath);
	}

	public void deleteTargetBinderWebDirectory(String sourceDirectoryPath) throws Exception
	{
		String targetBinderWebDirectoryPath = getTargetBinderWebDirectoryPath(sourceDirectoryPath);
		File targetBinderWebDirectoryFile = new File(targetBinderWebDirectoryPath);
		if (targetBinderWebDirectoryFile.exists() == false)
			return; // Do nothing
		if (Utilities.deleteDirectory(targetBinderWebDirectoryFile) == false)
			throw new BinderException(
					"Error in deleteTargetBinderWebDirectory:\n Cannot delete HTML Binder directory\n "
							+ targetBinderWebDirectoryPath);
	}

	private void processPartialBinderTreeRecursive(String sourceDirectoryPath) throws Exception
	{
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		File[] childrenFiles = sourceDirectoryFile.listFiles();
		for (int n = 0; n < childrenFiles.length; n++)
		{
			if (childrenFiles[n].isDirectory())
			{
				String name = childrenFiles[n].getName();
				if ((name.startsWith("_")) || (name.startsWith(".")))
					continue;
				String childDirectoryPath = childrenFiles[n].getPath();
				processPartialBinderTreeRecursive(childDirectoryPath);
				fDirectoryProc.processBinderDirectory(childDirectoryPath);
				Report.debug("Processing: " + childDirectoryPath);
			}
		}
	}

	public void processPartialBinderTree(String sourceDirectoryPath) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in processPartialBinderTree:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in processPartialBinderTree:\n Source Directory not found!");
		if (sourceDirectoryFile.isDirectory() == false)
			throw new BinderException("Error in processPartialBinderTree:\n sourcePath not a directory");
		processPartialBinderTreeRecursive(sourceDirectoryPath);
		fDirectoryProc.processBinderDirectory(sourceDirectoryPath);
		Report.debug("Processing: " + sourceDirectoryPath);
	}

	public void processAllBinderTree(String sourceDirectoryPath) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in processAllBinderTree:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in processAllBinderTree:\n Source Directory not found!");
		if (sourceDirectoryFile.isDirectory() == false)
			throw new BinderException("Error in processAllBinderTree:\n sourcePath not a directory");
		String sourceBinderDirectoryPath = System.getProperty("source.binder.directory.path");
		processPartialBinderTree(sourceBinderDirectoryPath);
	}

	private void cleanPartialBinderTreeRecursive(String sourceDirectoryPath, String excludedFileExtensions)
			throws Exception
	{
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		File[] childrenFiles = sourceDirectoryFile.listFiles();
		for (int n = 0; n < childrenFiles.length; n++)
		{
			if (childrenFiles[n].isDirectory())
			{
				String name = childrenFiles[n].getName();
				if ((name.startsWith("_")) || (name.startsWith(".")))
					continue;
				String childDirectoryPath = childrenFiles[n].getPath();
				cleanPartialBinderTreeRecursive(childDirectoryPath, excludedFileExtensions);
				fDirectoryProc.cleanDirectory(childDirectoryPath, excludedFileExtensions);
				Report.debug("Cleaning: " + childDirectoryPath);
			}
		}
	}

	public void cleanPartialBinderTree(String sourceDirectoryPath, String excludedFileExtensions) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in cleanPartialBinderTree:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in cleanPartialBinderTree:\n Source Directory not found!");
		if (sourceDirectoryFile.isDirectory() == false)
			throw new BinderException("Error in cleanPartialBinderTree:\n sourcePath not a directory");
		cleanPartialBinderTreeRecursive(sourceDirectoryPath, excludedFileExtensions);
		fDirectoryProc.cleanDirectory(sourceDirectoryPath, excludedFileExtensions);
		Report.debug("Cleaning: " + sourceDirectoryPath);
	}

	public void cleanAllBinderTree(String sourceDirectoryPath, String excludedFileExtensions) throws Exception
	{
		if (sourceDirectoryPath == null)
			throw new BinderException("Error in cleanAllBinderTree:\ndirectorySourcePath null");
		File sourceDirectoryFile = new File(sourceDirectoryPath);
		if (sourceDirectoryFile.exists() == false)
			throw new BinderException("Error in cleanAllBinderTree:\n Source Directory not found!");
		if (sourceDirectoryFile.isDirectory() == false)
			throw new BinderException("Error in cleanAllBinderTree:\n sourcePath not a directory");
		String sourceBinderDirectoryPath = System.getProperty("source.binder.directory.path");
		cleanPartialBinderTree(sourceBinderDirectoryPath, excludedFileExtensions);
	}

}
